﻿# Script to fix MySites when migrating to new domain
# The script does the following:
# Upgrade site to SP2013 if needed
# Update site owner:
#     - By reference in mapping file
#     - By replacing old domain name with new domain name
#
# Update site groups:
#     - By reference in mapping file
#     - By replacing old domain name with new domain name
#
# Updates user properties if they are invalid 
#     - PersonalSpace
#     - PersonalSiteCapabilities
#     - PersonalSiteInstantiationState
#
# (you might want to tweak the check of PersonalSpace update to force an update
#  if PersonalSpace has already has an Url but it's invalid)


# To work:
# Modify $siteToFix, $oldDomain, $newDomain to fit your environment
# CSV File in following format: (without domain name included)

# Old|New
# oldAcconut1|newAccount1
# oldAccount2|newAccount2

# // Jonas Widriksson, 2016

$siteToFix = "http://mysites.dev.local/personal/administrator"
$oldDomain = "Old_Domain_Name"
$newDomain = "New_Domain_Name"
$csvFile = "D:\AD_Mapping.csv"

Add-PSSnapin "Microsoft.SharePoint.PowerShell"


$webApplication = Get-SPWebApplication -Identity $siteToFix
$context = Get-SPServiceContext $webApplication.Sites[0]
$profileManager = new-object Microsoft.Office.Server.UserProfiles.UserProfileManager($context)
$farm = Get-SPFarm

$mappings = Import-Csv $csvFile -Header OldAccount, NewAccount -Delimiter '|' -Encoding UTF8

[Microsoft.SharePoint.SPSecurity]::RunWithElevatedPrivileges({    

#Iterate through site collection in My Site Host web application
foreach($siteCol in $webApplication.Sites | where { ($_.RootWeb.WebTemplateId -eq  21) } ) {
try{
	Write-Host "------ Start site ------ : " $siteCol.Url -ForegroundColor Cyan
	Write-Host "Site owner: " $siteCol.Owner.LoginName -ForegroundColor Gray

    # $keyb = Read-Host -Prompt "Waiting for key press..."

    # Check upgrade status
    if ($siteCol.CompatibilityLevel -eq '14') {
        Write-Host "Upgrading site..." -ForegroundColor Green
        upgrade-spsite $siteCol.Url -versionupgrade
        Write-Host "Upgrade complete!" -ForegroundColor Green
    }

    $siteOwnerLoginName = $siteCol.Owner.LoginName

    # Check site owner
    if ($siteCol.Owner -ilike "$oldDomain*") {
        Write-Host "Site owner not new domain... fixing... " $siteCol.Owner.LoginName -ForegroundColor Green

        $userMap = $mappings | Where-Object OldAccount -ilike  ($siteCol.Owner.LoginName -ireplace "$oldDomain\\", "")

        if ($userMap) {
            Write-Host "Found mapping: " $userMap.OldAccount " -> " $userMap.NewAccount -ForegroundColor Gray
            $newLogin = "$newDomain\" + ($userMap.NewAccount)
        }
        else {
            Write-Host "No mapping found. Trying to replace $oldDomain -> $newDomain" -ForegroundColor Gray
            $newLogin = ($siteCol.Owner -ireplace $oldDomain, $newDomain)
        }

        
        Set-SPSite -Identity $siteCol -OwnerAlias $newLogin

        $siteOwnerLoginName = $newLogin
    }

    # Check site groups/users
    $group = $siteCol.RootWeb.SiteUsers
    foreach ($grp in $group) {    
        if($grp.IsDomainGroup -eq "True") {            
            if($grp.LoginName -ilike "$oldDomain*") {                
                Write-Host "Group name: " $grp.Name
                Write-Host "Group LoginName: " $grp.LoginName

                $oldGroupName = $grp.LoginName    

                $userMap = $mappings | Where-Object OldAccount -ilike ($grp.LoginName -ireplace "$oldDomain\\", "")

                if ($userMap) {
                    Write-Host "Found group mapping: " $userMap.OldAccount " -> " $userMap.NewAccount -ForegroundColor Gray
                    $newLoginName = "$newDomain\" + ($userMap.NewAccount)
                }
                else {
                    Write-Host "No group mapping found. Trying to replace $oldDomain -> $newDomain" -ForegroundColor Gray
                    $newLoginName = ($oldGroupName -ireplace $oldDomain, $newDomain)
                }                               

                if ($oldGroupName -ne $newLoginName) {
                    Write-Host "Migrate group: " $newLoginName  -ForegroundColor Green
                    $farm.MigrateGroup($grp.LoginName, $newLoginName)
                    Write-Host "Migration group done!" -ForegroundColor Green
                }
            }
        }
    }

    # Check profile
	$uProfile = $profileManager.GetUserProfile($siteOwnerLoginName)
	
    if ($uProfile) {
        Write-Host "Found userprofile: " $uProfile.DisplayName -ForegroundColor Gray
		Write-Host "User personal site: " $uProfile.PersonalSite.ServerRelativeUrl -ForegroundColor Gray

        $updateProfile = $false
        
        if($uProfile.PersonalSite -eq $null)
        {
            #If link has been removed, update the user profile
            Write-Host -ForegroundColor Red "Personal Site is not linked"
            Write-Host -ForegroundColor Green "Updating the link" $siteCol.RootWeb.ServerRelativeUrl
 
            # Update personalspace
            $uProfile["PersonalSpace"].Value = $siteCol.RootWeb.ServerRelativeUrl
            $updateProfile = $true
        }

        # Check so that site is activated
        if ($uProfile["SPS-PersonalSiteCapabilities"].Value -eq $null -or $uProfile["SPS-PersonalSiteCapabilities"].Value -eq 0) {
            
            Write-Host -ForegroundColor Red "Personal Site has no capabilities"
            Write-Host -ForegroundColor Green "Updating capabilities" $siteCol.RootWeb.ServerRelativeUrl

            $uProfile["SPS-PersonalSiteCapabilities"].Value = 13

            Add-Content .\ErrorLog.txt "PersonalSiteCapabilities: 13, $siteCol.Url"

            $updateProfile = $true
        }

        # Check so that site is not marked as deleted
        if ($uProfile["SPS-PersonalSiteInstantiationState"].Value -eq $null -or $uProfile["SPS-PersonalSiteInstantiationState"].Value -eq 0 -or $uProfile["SPS-PersonalSiteInstantiationState"].Value -eq 3) {

            Write-Host -ForegroundColor Red "Site marked as deleted or not created"
            Write-Host -ForegroundColor Green "Updating site as created" $siteCol.RootWeb.ServerRelativeUrl

            $uProfile["SPS-PersonalSiteInstantiationState"].Value = 2

            Add-Content .\ErrorLog.txt "PersonalSiteInstantiationState: 2, $siteCol.Url"

            $updateProfile = $true
        }

        if ($updateProfile) {
            Write-Host "Commit changes to UPS..." -ForegroundColor Green
            $uProfile.Commit()
        }

    }
    else {
        Write-Host "No Profile found!" -ForegroundColor Red
    }
    Write-Host "----- End -----" -ForegroundColor Cyan

    $siteCol.Dispose

}
    catch{
        Write-Host "Error: " $_ -ForegroundColor Red
		$ownerLoginName = $siteCol.Owner.LoginName
		Add-Content .\ErrorLog.txt "User: $ownerLoginName $_"
        $siteCol.Dispose
	} 
    }
     
})
       